@@ -78,7 +78,7 @@ class Registry:
             session, database_name=self._database_name, schema_name=self._schema_name
         )
 
-        self.enable_monitoring = options.get("enable_monitoring", False) if options else False
+        self.enable_monitoring = options.get("enable_monitoring", True) if options else True
         if self.enable_monitoring:
             monitor_statement_params = telemetry.get_statement_params(
                 project=telemetry.TelemetryProject.MLOPS.value,
@@ -108,6 +108,7 @@ class Registry:
         metrics: Optional[Dict[str, Any]] = None,
         conda_dependencies: Optional[List[str]] = None,
         pip_requirements: Optional[List[str]] = None,
+        artifact_repository_map: Optional[Dict[str, str]] = None,
         target_platforms: Optional[List[model_types.SupportedTargetPlatformType]] = None,
         python_version: Optional[str] = None,
         signatures: Optional[Dict[str, model_signature.ModelSignature]] = None,
@@ -117,45 +118,64 @@ class Registry:
         options: Optional[model_types.ModelSaveOption] = None,
     ) -> ModelVersion:
         """
-        Log a model with various parameters and metadata.
+        Log a model with various parameters and metadata, or a ModelVersion object.
 
         Args:
-            model: Model object of supported types such as Scikit-learn, XGBoost, LightGBM, Snowpark ML,
-                PyTorch, TorchScript, Tensorflow, Tensorflow Keras, MLFlow, HuggingFace Pipeline,
-                Sentence Transformers, or Custom Model.
-            model_name: Name to identify the model.
+            model: Supported model or ModelVersion object.
+                - Supported model: Model object of supported types such as Scikit-learn, XGBoost, LightGBM, Snowpark ML,
+                PyTorch, TorchScript, Tensorflow, Tensorflow Keras, MLFlow, HuggingFace Pipeline, Sentence Transformers,
+                or Custom Model.
+                - ModelVersion: Source ModelVersion object used to create the new ModelVersion object.
+            model_name: Name to identify the model. This must be a valid Snowflake SQL Identifier. Alphanumeric
+                characters and underscores are permitted.
+                See https://docs.snowflake.com/en/sql-reference/identifiers-syntax for more.
             version_name: Version identifier for the model. Combination of model_name and version_name must be unique.
                 If not specified, a random name will be generated.
             comment: Comment associated with the model version. Defaults to None.
             metrics: A JSON serializable dictionary containing metrics linked to the model version. Defaults to None.
-            signatures: Model data signatures for inputs and outputs for various target methods. If it is None,
-                sample_input_data would be used to infer the signatures for those models that cannot automatically
-                infer the signature. Defaults to None.
-            sample_input_data: Sample input data to infer model signatures from.
-                It would also be used as background data in explanation and to capture data lineage. Defaults to None.
             conda_dependencies: List of Conda package specifications. Use "[channel::]package [operator version]" syntax
                 to specify a dependency. It is a recommended way to specify your dependencies using conda. When channel
                 is not specified, Snowflake Anaconda Channel will be used. Defaults to None.
             pip_requirements: List of Pip package specifications. Defaults to None.
-                Currently it is not supported since Model can only executed in Snowflake Warehouse where all
-                dependencies are required to be retrieved from Snowflake Anaconda Channel.
+                Models with pip requirements are currently only runnable in Snowpark Container Services.
+                See https://docs.snowflake.com/en/developer-guide/snowflake-ml/model-registry/container for more.
+                Models with pip requirements specified will not be executable in Snowflake Warehouse where all
+                dependencies must be retrieved from Snowflake Anaconda Channel.
+            artifact_repository_map: Specifies a mapping of package channels or platforms to custom artifact
+                repositories. Defaults to None. Currently, the mapping applies only to warehouse execution.
+                Note : This feature is currently in Private Preview; please contact your Snowflake account team
+                to enable it.
+                Format: {channel_name: artifact_repository_name}, where:
+                   - channel_name: The name of the Conda package channel (e.g., 'condaforge') or 'pip' for pip packages.
+                   - artifact_repository_name: The name or URL of the repository to fetch packages from.
             target_platforms: List of target platforms to run the model. The only acceptable inputs are a combination of
                 {"WAREHOUSE", "SNOWPARK_CONTAINER_SERVICES"}. Defaults to None.
             python_version: Python version in which the model is run. Defaults to None.
+            signatures: Model data signatures for inputs and outputs for various target methods. If it is None,
+                sample_input_data would be used to infer the signatures for those models that cannot automatically
+                infer the signature. If not None, sample_input_data should not be specified. Defaults to None.
+            sample_input_data: Sample input data to infer model signatures from.
+                It would also be used as background data in explanation and to capture data lineage. Defaults to None.
             code_paths: List of directories containing code to import. Defaults to None.
             ext_modules: List of external modules to pickle with the model object.
                 Only supported when logging the following types of model:
                 Scikit-learn, Snowpark ML, PyTorch, TorchScript and Custom Model. Defaults to None.
             options (Dict[str, Any], optional): Additional model saving options.
+
                 Model Saving Options include:
+
                 - embed_local_ml_library: Embed local Snowpark ML into the code directory or folder.
                     Override to True if the local Snowpark ML version is not available in the Snowflake Anaconda
                     Channel. Otherwise, defaults to False
-                - relax_version: Whether or not relax the version constraints of the dependencies when running in the
+                - relax_version: Whether to relax the version constraints of the dependencies when running in the
                     Warehouse. It detects any ==x.y.z in specifiers and replaced with >=x.y, <(x+1). Defaults to True.
                 - function_type: Set the method function type globally. To set method function types individually see
-                  function_type in model_options.
-                - method_options: Per-method saving options including:
+                    function_type in model_options.
+                - method_options: Per-method saving options. This dictionary has method names as keys and dictionary
+                    values with the desired options.
+
+                    The following are the available method options:
+
                     - case_sensitive: Indicates whether the method and its signature should be case sensitive.
                         This means when you refer the method in the SQL, you need to double quote it.
                         This will be helpful if you need case to tell apart your methods or features, or you have
@@ -163,7 +183,10 @@ class Registry:
                     - max_batch_size: Maximum batch size that the method could accept in the Snowflake Warehouse.
                         Defaults to None, determined automatically by Snowflake.
                     - function_type: One of supported model method function types (FUNCTION or TABLE_FUNCTION).
+        Returns:
+            ModelVersion: ModelVersion object corresponding to the model just logged.
         """
+
         ...
 
     @overload
@@ -195,6 +218,7 @@ class Registry:
             "metrics",
             "conda_dependencies",
             "pip_requirements",
+            "artifact_repository_map",
             "target_platforms",
             "python_version",
             "signatures",
@@ -210,10 +234,12 @@ class Registry:
         metrics: Optional[Dict[str, Any]] = None,
         conda_dependencies: Optional[List[str]] = None,
         pip_requirements: Optional[List[str]] = None,
+        artifact_repository_map: Optional[Dict[str, str]] = None,
         target_platforms: Optional[List[model_types.SupportedTargetPlatformType]] = None,
         python_version: Optional[str] = None,
         signatures: Optional[Dict[str, model_signature.ModelSignature]] = None,
         sample_input_data: Optional[model_types.SupportedDataType] = None,
+        user_files: Optional[Dict[str, List[str]]] = None,
         code_paths: Optional[List[str]] = None,
         ext_modules: Optional[List[ModuleType]] = None,
         task: model_types.Task = model_types.Task.UNKNOWN,
@@ -228,25 +254,38 @@ class Registry:
                 PyTorch, TorchScript, Tensorflow, Tensorflow Keras, MLFlow, HuggingFace Pipeline, Sentence Transformers,
                 or Custom Model.
                 - ModelVersion: Source ModelVersion object used to create the new ModelVersion object.
-            model_name: Name to identify the model.
+            model_name: Name to identify the model. This must be a valid Snowflake SQL Identifier. Alphanumeric
+                characters and underscores are permitted.
+                See https://docs.snowflake.com/en/sql-reference/identifiers-syntax for more.
             version_name: Version identifier for the model. Combination of model_name and version_name must be unique.
                 If not specified, a random name will be generated.
             comment: Comment associated with the model version. Defaults to None.
             metrics: A JSON serializable dictionary containing metrics linked to the model version. Defaults to None.
+            conda_dependencies: List of Conda package specifications. Use "[channel::]package [operator version]" syntax
+                to specify a dependency. It is a recommended way to specify your dependencies using conda. When channel
+                is not specified, Snowflake Anaconda Channel will be used. Defaults to None.
+            pip_requirements: List of Pip package specifications. Defaults to None.
+                Models with pip requirements are currently only runnable in Snowpark Container Services.
+                See https://docs.snowflake.com/en/developer-guide/snowflake-ml/model-registry/container for more.
+                Models with pip requirements specified will not be executable in Snowflake Warehouse where all
+                dependencies must be retrieved from Snowflake Anaconda Channel.
+            artifact_repository_map: Specifies a mapping of package channels or platforms to custom artifact
+                repositories. Defaults to None. Currently, the mapping applies only to warehouse execution.
+                Note : This feature is currently in Private Preview; please contact your Snowflake account team to
+                enable it.
+                Format: {channel_name: artifact_repository_name}, where:
+                   - channel_name: The name of the Conda package channel (e.g., 'condaforge') or 'pip' for pip packages.
+                   - artifact_repository_name: The name or URL of the repository to fetch packages from.
+            target_platforms: List of target platforms to run the model. The only acceptable inputs are a combination of
+                {"WAREHOUSE", "SNOWPARK_CONTAINER_SERVICES"}. Defaults to None.
+            python_version: Python version in which the model is run. Defaults to None.
             signatures: Model data signatures for inputs and outputs for various target methods. If it is None,
                 sample_input_data would be used to infer the signatures for those models that cannot automatically
                 infer the signature. If not None, sample_input_data should not be specified. Defaults to None.
             sample_input_data: Sample input data to infer model signatures from.
                 It would also be used as background data in explanation and to capture data lineage. Defaults to None.
-            conda_dependencies: List of Conda package specifications. Use "[channel::]package [operator version]" syntax
-                to specify a dependency. It is a recommended way to specify your dependencies using conda. When channel
-                is not specified, Snowflake Anaconda Channel will be used. Defaults to None.
-            pip_requirements: List of Pip package specifications. Defaults to None.
-                Currently it is not supported since Model can only executed in Snowflake Warehouse where all
-                dependencies are required to be retrieved from Snowflake Anaconda Channel.
-            target_platforms: List of target platforms to run the model. The only acceptable inputs are a combination of
-                {"WAREHOUSE", "SNOWPARK_CONTAINER_SERVICES"}. Defaults to None.
-            python_version: Python version in which the model is run. Defaults to None.
+            user_files: Dictionary where the keys are subdirectories, and values are lists of local file name
+                strings. The local file name strings can include wildcards (? or *) for matching multiple files.
             code_paths: List of directories containing code to import. Defaults to None.
             ext_modules: List of external modules to pickle with the model object.
                 Only supported when logging the following types of model:
@@ -261,11 +300,15 @@ class Registry:
                 - embed_local_ml_library: Embed local Snowpark ML into the code directory or folder.
                     Override to True if the local Snowpark ML version is not available in the Snowflake Anaconda
                     Channel. Otherwise, defaults to False
-                - relax_version: Whether or not relax the version constraints of the dependencies when running in the
+                - relax_version: Whether to relax the version constraints of the dependencies when running in the
                     Warehouse. It detects any ==x.y.z in specifiers and replaced with >=x.y, <(x+1). Defaults to True.
                 - function_type: Set the method function type globally. To set method function types individually see
                   function_type in model_options.
-                - method_options: Per-method saving options including:
+                - method_options: Per-method saving options. This dictionary has method names as keys and dictionary
+                    values with the desired options. See the example below.
+
+                    The following are the available method options:
+
                     - case_sensitive: Indicates whether the method and its signature should be case sensitive.
                         This means when you refer the method in the SQL, you need to double quote it.
                         This will be helpful if you need case to tell apart your methods or features, or you have
@@ -276,6 +319,28 @@ class Registry:
 
         Returns:
             ModelVersion: ModelVersion object corresponding to the model just logged.
+
+        Example::
+
+            from snowflake.ml.registry import Registry
+
+            # create a session
+            session = ...
+
+            registry = Registry(session=session)
+
+            # Define `method_options` for each inference method if needed.
+            method_options={
+              "predict": {
+                "case_sensitive": True
+              }
+            }
+
+            registry.log_model(
+              model=model,
+              model_name="my_model",
+              method_options=method_options,
+            )
         """
         statement_params = telemetry.get_statement_params(
             project=_TELEMETRY_PROJECT,
@@ -297,10 +362,12 @@ class Registry:
             metrics=metrics,
             conda_dependencies=conda_dependencies,
             pip_requirements=pip_requirements,
+            artifact_repository_map=artifact_repository_map,
             target_platforms=target_platforms,
             python_version=python_version,
             signatures=signatures,
             sample_input_data=sample_input_data,
+            user_files=user_files,
             code_paths=code_paths,
             ext_modules=ext_modules,
             task=task,
